#!/usr/bin/perl -w
# template.mod.pl
# ----------------
#
# Copyright (C) 2001-2002 by Duke University
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
#
# To view the help for this module, run "perldoc template.mod.pl"
# To create the text version, run
#      "perldoc -t template.mod.pl > template.mod.txt"
# For help with POD (plain old documentation) see the "perlpod"
# man page.  POD is really easy, so check it out!
#
# $Id: template.mod.pl,v 1.8 2002/02/14 03:26:42 mstenner Exp $
#
# @Author  Michael D. Stenner <mstenner@phy.duke.edu>
#          Konstantin Riabitsev <icon@phy.duke.edu>
# @version $Date: 2002/02/14 03:26:42 $
#

##
# Strict enforces good coding practices by making you observe the
# variable scope.
# Add other packages, too.
#
use strict;

##
# Identify ourselves (used in the mlog subroutine).
#
my $module_name = "template.mod";

#####################################################################
=pod

=head1 NAME

 template.mod.pl    module template for dulog
 Version            %VERSION%  date
 Author             Your name <email@address.com>

=head1 DESCRIPTION

This is a template for dulog modules in perl.  Edit as you like.  Subroutines
will be put here is they are created and if they might be generally useful.
Just delete the ones you do not use.

=head1 OPTIONS

This module accepts a number of options.  All options are read from
environment variables.  These options can be set in the dulog config
file (/etc/dulog/dulog.conf by default).  Here are the options and
their default values:

=head2 GENERAL DULOG OPTIONS

These options are officially sanctioned by dulog and are common to
most modules.

=over 5

=item DEBUG => unset

When set, this module (and probably all of the others) spit debugging
info to STDOUT.

=item QUIET => unset

When set, this module (and probably all of the others) spit out a
minimal amount of info to STDOUT.  Note that this module may still
write to STDOUT if there is a serious problem that may result in log
loss.  QUIET mode overrides DEBUG mode.

=item LOGCAT => unset (read from STDIN)

This variable is set by dulog and is the file containing logs that
should be read by the module.  If unset, this module reads logs from
STDIN.

=item LOGREPORT => unset (write to STDOUT)

This variable is set by dulog and is the file to which the final
report should be written.  If unset, this module writes the report to
STDOUT.

=item LOGFILTER => unset (write to STDERR)

This variable is set by dulog and is the file to which the lines
processed by your module should be written. Dulog will fgrep this
file against the log report before mailing it (if configured to do
so). If unset, this module will output to STDERR.

=item CONFDIR => '../etc'

Location where configuration files are looked for.  NOTE: don't be
alarmed by the default value.  It's that way for easy testing and
dulog overrides it.

=back

=cut
#####################################################################

##
# General dulog options. These shouldn't change from module to module.
#
my $debug       = option('DEBUG',            undef); # dulog in debug mode
my $quiet       = option("QUIET",            undef); # dulog in quiet mode
my $input       = option("LOGCAT",           undef); # file to read
my $output      = option("LOGREPORT",        undef); # file to write report to
my $filter      = option("LOGFILTER",        undef); # file with proc'd str's
my $confdir     = option("CONFDIR",          "../etc"); # where to find config

#####################################################################
=pod

=head2 MODULE-SPECIFIC OPTIONS

These options are specific to this module.

=cut
#####################################################################

##
# Put any specific options used by this module here.
#

#####################################################################
=pod

=head1 CUSTOMIZING THIS MODULE

Tell people about tweaking your module here!

=cut
#####################################################################

##
# Set up logging level.
# NOTE: logs at level 0 (serious problems) are reported even when 
# "QUIET" is on!
#
my $loglevel = (defined($debug)) ? 2 : 1;
$loglevel = 0 if defined($quiet);
*LOGOUT = *STDOUT;

##
# Open the input file, $LOGCAT or read from stdin if LOGCAT isn't
# defined.
#
if (defined($input)) {
    open(INPUT, $input) or mlog(0, "cannot open input file $input\n");
} else {
    *INPUT = *STDIN;
}

##
# Open output file, $LOGREPORT or write to STDOUT if LOGREPORT
# isn't defined.
#
if (defined($output)) {
    open(OUTPUT, ">$output") or mlog(0, "cannot open output file $output\n");
} else {
    *OUTPUT = *STDOUT;
}

##
# Open the file for processed strings, or write to STDERR if 
# LOGFILTER is not defined.
#
if (defined($filter)){
    open(FILTER, ">$filter") or mlog(0, "cannot open filtered strings file\n");
} else {
    *FILTER = *STDERR;
}

##
# Define any global variables here.
# @filtered array is where all processed strings go.
#
my @filtered;

##
# Main module code.
#
mlog(1, "beginning to read input");
my $line;

##
# Iterate through all lines in the logs and analyze the ones that
# match. Analyzed and processed lines get pushed into the @filtered
# array so they can be later weeded by dulog.
#
while (defined($line = <INPUT>)) {
    ##
    # Put line-analyzing code here.
    #
}
mlog(1, "finished reading input");

##
# If @filtered array is non-empty, then place the output into the
# LOGREPORT, and write the filtered strings into LOGFILTER.
#
if ($#filtered >= 0){
    mlog(1, "generating the report");
    ##
    # Put your report-formatting code here.
    #

    ##
    # Put all processed strings into LOGFILTER.
    print FILTER join("", @filtered);
    mlog(1, "done generating report");
}

mlog(1, "Exiting...");

###################################################################
# subroutines

##
# Since all syslog lines start uniformly, use this sub to
# grab the name of the system from the log line.
#
# @param  $1 The log line.
# @return    The name of the system this log line refers to.
#
sub systemname {
    my $line = shift;
    (my $system) = $line =~ m/.{15}\s(\S*)\s.*/;
    ##
    # syslog-ng can report hosts in a more complicated way :)
    #
    if ($system =~ m{[@/](\S+)}) { $system = $1; }
    return($system);
}

##
# This sub is used for logging debug data. The logging levels are
# as follows:
# level 0 -- serious problem, should always be reported
#            (logs are probably not being reported correctly)
# level 1 -- notification of important events
# level 2 -- debugging output, useful only when coding this module
#
# Unless the log level is 0, nothing is written out to the console if
# QUIET env variable is set.
#
# @param  $1 logging level.
# @param  $2 Message to output.
# @return    void.
#
sub mlog {
    my $level = shift;
    print(LOGOUT "DULOG($module_name): ", @_, "\n") if ($level <= $loglevel);
}

##
# A wrapper to process the options passed in by environment variables.
# If the referred ENV variable is unset, then return the default value.
# This behavior is useful for debugging the module.
#
# @param  $1 The name of the ENV variable to grab.
# @param  $2 The default value to return if the ENV variable is unset.
# @return    The value of the environment variable, or the default value
#            if the variable is unset.
#
sub option {
    my $op      = shift;
    my $default = shift;
    return(exists($ENV{$op}) ? $ENV{$op} : $default);
}

##
# A wrapper to create the separator header within the report. Currently:
#
# #
# # MESSAGE
# #
#
# @param  $1 The message content.
# @return    The header formatted accordingly.
#
sub mkreporthdr {
    my $msg = shift;
    my $hdr = "#\n# $msg\n#\n";
    return $hdr;
}
